@@ -35,6 +35,7 @@
 #include "settings.h"
 #include "undo.hpp"
 #include "callbacks.h"
+#include "netrender.hpp"
 
 using namespace std;
 
@@ -58,8 +59,12 @@ double start_time = 0.0;
 bool isRendering = false;
 bool isPostRendering = false;
 
+bool newLineRendered = false;
+
 cImage mainImage(800, 600);
 
+bool *linesToSend = NULL;
+
 double real_clock(void)
 {
 #ifdef WIN32 /* WINDOWS */
@@ -118,6 +123,8 @@ void *MainThread(void *ptr)
 	int AO_quality = param.globalIlumQuality;
 
 
+
+
 	bool fractColor = param.imageSwitches.coloringEnabled;
 	cTexture *envmap_texture = param.envmapTexture;
 	cTexture *lightmap_texture = param.lightmapTexture;
@@ -226,29 +233,42 @@ void *MainThread(void *ptr)
 	double tileXOffset = (tile % tiles);
 	double tileYOffset = (tile / tiles);
 
+	bool breakX = false;
+
+	//calcParam.doubles.fakeLightsOrbitTrap.x = 5;
+	//calcParam.doubles.fakeLightsOrbitTrap.y = 0;
+	//calcParam.doubles.fakeLightsOrbitTrap.z = 0;
+
 	//2-pass loop (for multi-threading)
-	for (int pass = 0; pass < 2; pass++)
+	for (int pass = 0; pass < 3; pass++)
 	{
-		if (pass == 1) start = 0;
+		if (pass > 0) start = 0;
 
 		//main loop for z values
 		for (z = start; z <= height - progressive; z += progressive)
 		{
 			//checking if some another thread is not rendering the same z
-			if (parametry->thread_done[z] == 0)
+			if (parametry->thread_done[z] == 0 && !breakX)
 			{
 				//giving information for another threads that this thread renders this z value
 				parametry->thread_done[z] = thread_number + 1;
 
+				newLineRendered = true;
+
 				//WriteLogDouble("Started rendering line", z);
 
 				//main loop for x values
+				breakX = false;
 				for (int x = 0; x <= width - progressive; x += progressive)
 				{
 					//checking if program was not closed
 					if (programClosed)
 					{
 						delete[] vectorsAround;
+						delete[] distanceBuff;
+						delete[] stepBuff;
+						delete[] distanceBuffRefl;
+						delete[] stepBuffRefl;
 						return NULL;
 					}
 
@@ -261,10 +281,17 @@ void *MainThread(void *ptr)
 					if (y_scan_start > min_y) min_y = y_scan_start;
 
 					double x2,z2;
+					bool hemisphereCut = false;
 					if (perspectiveType == fishEye || perspectiveType == equirectangular)
 					{
-						x2 = M_PI * ((double) x / width / tiles - 0.5 + tileXOffset/tiles) * aspectRatio;
-						z2 = M_PI * ((double) z / height / tiles  - 0.5 + tileYOffset/tiles);
+						x2 = ((double) x / width / tiles - 0.5 + tileXOffset/tiles) * aspectRatio;
+						z2 = ((double) z / height / tiles  - 0.5 + tileYOffset/tiles);
+						if(param.fishEyeCut && sqrt(x2*x2 + z2*z2) > 0.5 / fov)
+						{
+								hemisphereCut = true;
+						}
+						x2*=M_PI;
+						z2*=M_PI;
 					}
 					else
 					{
@@ -300,11 +327,13 @@ void *MainThread(void *ptr)
 					calcParam.specialColour = 0;
 					int specialColour = 0;
 
+					double fakeLightVisible = 0.0;
+
 					//main loop for y values (depth)
 					for (double y = min_y; y < max_y; y += stepYpersp)
 					{
 						//recheck threads
-						if (parametry->thread_done[z] == thread_number + 1)
+						if (parametry->thread_done[z] == thread_number + 1 || parametry->thread_done[z] == 0)
 						{
 
 							//perspective factor (wsp = factor in Polish :-)
@@ -414,6 +443,12 @@ void *MainThread(void *ptr)
 									stepYpersp = stepYpersp * (1.0 - Random(1000)/10000.0);
 
 									stepBuff[buffCount] = stepYpersp * zoom;
+									if(param.fakeLightsEnabled)
+									{
+										double r = Compute<orbitTrap>(point, calcParam);
+										double fakeLight = 1.0/(pow(r,10.0/param.doubles.fakeLightsVisibilitySize)*pow(10.0, 10.0/param.doubles.fakeLightsVisibilitySize) + 1e-100);
+										fakeLightVisible += fakeLight * stepYpersp * zoom;
+									}
 									buffCount++;
 								}
 
@@ -512,9 +547,25 @@ void *MainThread(void *ptr)
 								lastDist = dist;
 								lastStepY = stepYpersp;
 								lastCorrection = correction;
+
+								if (hemisphereCut)
+								{
+									found = false;
+									y_start = y;
+									last_distance = dist;
+									break;
+								}
 							}
 						}
-					} //next y
+						else
+						{
+							breakX = true;
+							//printf("BreakX, line = %d, thread = %d, thread_done[z] = %d\n", z, thread_number, parametry->thread_done[z]);
+							break;
+						}
+					}//next y
+
+					if(breakX) break;
 
 					//if(thread_number == 0) printf("next y\n");
 
@@ -602,6 +653,11 @@ void *MainThread(void *ptr)
 						//calculate shading from auxiliary lights
 						shadeAux = AuxLightsShader(&param, &calcParam, point, vn, viewVector, wsp_persp, dist_thresh, &shadeAuxSpec);
 
+						if(param.fakeLightsEnabled)
+						{
+							shadeAux = FakeLights(&param, &calcParam, point, vn, viewVector, dist_thresh, &shadeAuxSpec);
+						}
+
 						//calculate environment mapping reflection effect
 						envMapping = EnvMapping(vn, viewVector, envmap_texture);
 
@@ -619,7 +675,7 @@ void *MainThread(void *ptr)
 
 							double fog_visibility = pow(10, param.doubles.imageAdjustments.fogVisibility / 10 - 2.0)*zoom;
 
-							envMapping = (sShaderOutput){0,0,0};
+							envMapping.R = envMapping.G = envMapping.B = 0;
 							sShaderOutput shadeBuff[10];
 							sShaderOutput shadowBuff[10];
 							sShaderOutput specularBuff[10];
@@ -665,9 +721,9 @@ void *MainThread(void *ptr)
 										vnTemp = CalculateNormals(&param, &calcParam, pointTemp, dist_thresh);
 										shadeBuff[i] = MainShading(vnTemp, lightVector);
 										specularBuff[i] = MainSpecular(vnTemp, lightVector, viewTemp);
-										shadowBuff[i] = (sShaderOutput){1.0,1.0,1.0};
+										shadowBuff[i].R = shadowBuff[i].G = shadowBuff[i].B = 1.0;
 										if (shadow) shadowBuff[i] = MainShadow(&param, &calcParam, pointTemp, lightVector, wsp_persp, dist_thresh);
-										ambientBuff[i]=(sShaderOutput){0,0,0};
+										ambientBuff[i].R = ambientBuff[i].G = ambientBuff[i].B = 0;
 										if (global_ilumination)
 										{
 											if (fastGlobalIllumination)
@@ -682,7 +738,8 @@ void *MainThread(void *ptr)
 
 										int colorIndexTemp = SurfaceColour(&calcParam, pointTemp);
 										if (specialColour != 0) colorIndexTemp = specialColour*256;
-										sRGB color = { 256, 256, 256 };
+										sRGB color;
+										color.R = color.G = color.B = 256;
 										if (param.imageSwitches.coloringEnabled)
 										{
 											int color_number;
@@ -696,7 +753,9 @@ void *MainThread(void *ptr)
 											}
 											color = image->IndexToColour(color_number);
 										}
-										colorBuff[i] = (sShaderOutput){color.R/256.0,color.G/256.0,color.B/256.0};
+										colorBuff[i].R = color.R/256.0;
+										colorBuff[i].G = color.G/256.0;
+										colorBuff[i].B = color.B/256.0;
 
 										totalFog += scan;
 										fogBuff[i] = totalFog;;
@@ -846,8 +905,15 @@ void *MainThread(void *ptr)
 						}
 
 						image->PutPixelColor(x, z, colorIndex);
-						image->PutPixelZBuffer(x, z, y);
 
+						if (perspectiveType == fishEye || perspectiveType == equirectangular)
+						{
+							image->PutPixelZBuffer(x, z, y*1.0e12);
+						}
+						else
+						{
+							image->PutPixelZBuffer(x, z, y);
+						}
 						pixelData.shadowsBuf16 = shadow16;
 						pixelData.ambientBuf16 = globalLight;
 						pixelData.shadingBuf16 = shade16;
@@ -890,7 +956,8 @@ void *MainThread(void *ptr)
 					pixelData.glowBuf16 = counter;
 
 					//************* volumetric fog / light
-					sShaderOutput volFog = (sShaderOutput){ 0.0,0.0,0.0};
+					sShaderOutput volFog;
+					volFog.R = volFog.G = volFog.B = 0.0;
 					if (!param.imageSwitches.iterFogEnabled)
 					{
 						if (param.imageSwitches.volumetricLightEnabled)
@@ -901,15 +968,34 @@ void *MainThread(void *ptr)
 						{
 							double density;
 							volFog = VolumetricFog(&param, buffCount, distanceBuff, stepBuff, &density);
+
+							if(param.fakeLightsEnabled)
+							{
+								volFog.R += fakeLightVisible * 10000.0 * param.doubles.fakeLightsVisibility;
+								volFog.G += fakeLightVisible * 10000.0 * param.doubles.fakeLightsVisibility;
+								volFog.B += fakeLightVisible * 10000.0 * param.doubles.fakeLightsVisibility;
+								density += fakeLightVisible * param.doubles.fakeLightsVisibility;
+							}
 							pixelData.fogDensity16 = 65535 * density / (1.0 + density);
 						}
 					}
 					else
 					{
 						//iteration fog
+
+						unsigned short alpha2 = 0;
+						float zBuf = image->GetPixelZBuffer(x,z);
+						unsigned short colorIndex = image->GetPixelColor(x,z);
+						double fog_visibility = pow(10, param.doubles.imageAdjustments.fogVisibility / 10 - 2.0);
+						double fog_visibility_front = pow(10,  param.doubles.imageAdjustments.fogVisibilityFront / 10 - 2.0) - 10.0;
+						pixelData.volumetricLight.R = 0;
+						pixelData.volumetricLight.G = 0;
+						pixelData.volumetricLight.B = 0;
+						pixelData.fogDensity16 = 0;
+						sRGB16 oldPixel16 = image->CalculatePixel(pixelData, alpha2, zBuf, colorIndex, fog_visibility, fog_visibility_front);
 						double density;
-						volFog = IterationFog(&param, &calcParam, CVector3(x2, y, z2), y, min_y, last_distance, zoom, lightVector, found, &density, vectorsAround, vectorsCount);
-						pixelData.fogDensity16 = density * 65535;
+						volFog = IterationFog(&param, &calcParam, CVector3(x2, y, z2), y, min_y, last_distance, zoom, lightVector, found, &density, vectorsAround, vectorsCount, oldPixel16);
+						pixelData.fogDensity16 = 65535;
 						//end of iteration fog
 					}
 
@@ -941,6 +1027,8 @@ void *MainThread(void *ptr)
 
 				}//next x
 
+				if(!breakX) linesToSend[z] = true;
+
 				if(!image->IsLowMemMode()) image->Squares(z, progressive);
 
 				(*parametry->done)++;
@@ -957,6 +1045,16 @@ void *MainThread(void *ptr)
 				{
 					percent_done = (((double) *parametry->done / height) * 3.0 / 4.0 / progressive + progressiveDone) * 100.0;
 				}
+
+				if(param.noOfTiles > 1)
+				{
+					percent_done = (param.tileCount + percent_done/100.0) / (param.noOfTiles * param.noOfTiles)*100.0;
+				}
+				if(Interface_data.animMode)
+				{
+					percent_done = (param.fractal.frameNo - param.startFrame + percent_done/100.0) / (param.endFrame - param.startFrame + 1) * 100.0;
+				}
+
 				double time = real_clock() - start_time;
 				double time_to_go = (100.0 - percent_done) * time / percent_done;
 				int togo_time_s = (int) time_to_go % 60;
@@ -985,7 +1083,44 @@ void *MainThread(void *ptr)
 
 				//WriteLogDouble("Rendering line finished", z);
 			}//end if thread done
+			else
+			{
+				breakX = false;
+				if(pass < 2)
+				{
+					int firstFree = -1;
+					int lastFree = -1;
+					int maxHole = 0;
+					for(int i=0; i<height; i+=progressive)
+					{
+						if(firstFree < 0 && parametry->thread_done[i] == 0)
+						{
+							firstFree = i;
+						}
+						if(firstFree > 0 && parametry->thread_done[i] > 0)
+						{
+							lastFree = i;
 
+							int holeSize = lastFree - firstFree;
+							if (holeSize > maxHole)
+							{
+								z = (((firstFree+lastFree) / 2)/progressive) * progressive - progressive;
+								maxHole = holeSize;
+							}
+							firstFree = -1;
+							lastFree = -1;
+						}
+					}
+					if(firstFree > 0 && lastFree < 0)
+					{
+						z = (((firstFree+height) / 2)/progressive) * progressive;
+						maxHole = 1;
+					}
+					if(maxHole == 0) break;
+				}
+
+				if(programClosed) break;
+			}
 		}//next z
 	}// next pass
 	WriteLogDouble("Thread finished", thread_number);
@@ -1004,6 +1139,8 @@ void Render(sParamRender param, cImage *image, GtkWidget *outputDarea)
 
 	programClosed = false;
 	//getting image resolution from parameters
+	CheckPrameters(&param);
+
 	int width = image->GetWidth();
 	int height = image->GetHeight();
 
@@ -1012,6 +1149,41 @@ void Render(sParamRender param, cImage *image, GtkWidget *outputDarea)
 		//turn off refreshing if resolution is very low
 		bool bo_refreshing = true;
 
+		//sending settings to clients
+		if(netRender->IsServer()) SendSettingsToClients(param);
+
+		//if it is a server, send "run" command to all clients
+		if (netRender->IsServer())
+		{
+			for (int i = 0; i < netRender->getNoOfClients(); i++)
+			{
+				netRender->sendDataToClient(NULL, 0, "run", i, param.fractal.frameNo);
+			}
+		}
+
+		//calculate total number of vailable CPUs
+		int netCpuCount = NR_THREADS;
+		for(int i=0; i<netRender->getNoOfClients(); i++)
+		{
+			netCpuCount += netRender->getCpuCount(i);
+		}
+
+		//setting first lines to render
+		int cpuIndex = NR_THREADS;
+		if (netRender->IsServer())
+		{
+			for (int clientIndex = 0; clientIndex < netRender->getNoOfClients(); clientIndex++)
+			{
+				for(int i=0; i<netRender->getCpuCount(clientIndex); i++)
+				{
+					int startLine = cpuIndex * height / netCpuCount;
+					netRender->sendDataToClient(&startLine, sizeof(startLine), "stl", clientIndex, 0);
+					printf("set start line = %d\n", startLine);
+					cpuIndex++;
+				}
+			}
+		}
+
 		//clearing histogram tables
 		for (int i = 0; i < 64; i++)
 			histogram[i] = 0;
@@ -1027,22 +1199,31 @@ void Render(sParamRender param, cImage *image, GtkWidget *outputDarea)
 		Missed_DE_counter = 0;
 		DEerror = 0;
 		DE_counterForDEerror = 0;
-		start_time = real_clock();
 
 		//initialising threads
 		int *thread_done = new int[height];
-		GThread *Thread[NR_THREADS + 1];GError
-		*err[NR_THREADS + 1];sParam
-		thread_param[NR_THREADS];
+		int *thread_done_from_server = new int[height];
+		linesToSend = new bool[height];
+		GThread **Thread = new GThread *[NR_THREADS + 1];
+		GError **err = new GError *[NR_THREADS + 1];
+		sParam *thread_param = new sParam[NR_THREADS];
+
+		//initialise send/receive buffer
+		sAllImageData *lineOfImage = NULL;
+		sAllImageDataLowMem *lineOfImageLowMem = NULL;
+		if(!image->IsLowMemMode())
+			lineOfImage = new sAllImageData[width+1];
+		else
+			lineOfImageLowMem = new sAllImageDataLowMem[width+1];
 
 		int progressiveStart = 8;
-		if (param.recordMode || noGUI) progressiveStart = 1;
+		if (param.recordMode || noGUI || netRender->IsServer() || netRender->IsClient()) progressiveStart = 1;
 		image->progressiveFactor = progressiveStart;
 
-		int refresh_index = 0;
-		int refresh_skip = 1;
+		double refresh_skip = 0.1;
+		double refreshEndTime = real_clock();
 
-		if (image->IsLowMemMode()) image->CalculateGammaTable();
+		image->CalculateGammaTable();
 
 		for (int progressive = progressiveStart; progressive != 0; progressive /= 2)
 		{
@@ -1057,6 +1238,8 @@ void Render(sParamRender param, cImage *image, GtkWidget *outputDarea)
 			for (int i = 0; i < height; i++)
 			{
 				thread_done[i] = 0;
+				thread_done_from_server[i] = 0;
+				linesToSend[i] = false;
 			}
 			WriteLog("Thread data prepared");
 
@@ -1064,7 +1247,25 @@ void Render(sParamRender param, cImage *image, GtkWidget *outputDarea)
 			for (int i = 0; i < NR_THREADS; i++)
 			{
 				//sending some parameters to thread
-				thread_param[i].start = i * height / NR_THREADS;
+				if(!netRender->IsClient())
+				{
+					thread_param[i].start = i * height / netCpuCount;
+				}
+				else
+				{
+					//getting indexes of first lines to render
+					int startLine = 0;
+					char command[4];
+					size_t bytes_received = netRender->receiveDataFromServer(command);
+					if(!strcmp(command, "stl") && bytes_received == sizeof(int))
+					{
+						netRender->GetData(&startLine);
+					}
+					printf("command = %s, start line = %d\n", command, startLine);
+					thread_param[i].start = startLine;
+				}
+
+				//initialising local threads
 				thread_param[i].z = i;
 				thread_param[i].param = param;
 				thread_param[i].image = image;
@@ -1079,97 +1280,255 @@ void Render(sParamRender param, cImage *image, GtkWidget *outputDarea)
 			}
 
 			//refresh GUI
-			if (!noGUI && bo_refreshing)
+			if ((!noGUI && bo_refreshing) || netRender->IsClient())
 			{
-				if (image->IsPreview())
+				if (image->IsPreview() || netRender->IsClient())
 				{
 					while (gtk_events_pending())
 						gtk_main_iteration();
 				}
 
+
+				int done2 = 0;
 				//refreshing image and histograms during rendering
-				while (done < height / progressive - 1 && !programClosed)
+				while (done2 < height / progressive - 1 && !programClosed)
 				{
-					g_usleep(100000);
-					if (progressive < progressiveStart)
+					//send list of rendered lines
+					if (netRender->IsServer() && newLineRendered)
 					{
-						refresh_index++;
-
-						if (refresh_index >= refresh_skip)
+						newLineRendered = false;
+						for (int i = 0; i < netRender->getNoOfClients(); i++)
 						{
-							double refreshStartTime = real_clock();
+							netRender->sendDataToClient(thread_done, sizeof(int)*height, "lst", i, param.fractal.frameNo);
+						}
+					}
 
-							if (image->IsPreview())
+					char command[4];
+					//get rendered line if available
+					if (netRender->IsServer())
+					{
+						for (int i = 0; i < netRender->getNoOfClients(); i++)
+						{
+							netRender->sendDataToClient(NULL, 0, "get", i, param.fractal.frameNo);
+						}
+
+						for (int i = 0; i < netRender->getNoOfClients(); i++)
+						{
+							size_t bytes_recvd = netRender->receiveDataFromClient(command, i, param.fractal.frameNo);
+							if(!strcmp(command, "GET"))
 							{
-								param.SSAOEnabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(Interface.checkSSAOEnabled));
-								if (param.SSAOEnabled && !image->IsLowMemMode())
+								if(bytes_recvd > 0)
 								{
-									param.SSAOQuality = gtk_adjustment_get_value(GTK_ADJUSTMENT(Interface.adjustmentSSAOQuality));
-									PostRendering_SSAO(image, param.doubles.persp, param.SSAOQuality / 2, param.perspectiveType, param.quiet);
-									WriteLog("SSAO rendered");
-								}
-								image->CompileImage();
-								WriteLog("Image compiled");
+									int *last;
+									if(!image->IsLowMemMode())
+									{
+										netRender->GetData(lineOfImage);
+										last = (int*)&lineOfImage[width];
+									}
+									else
+									{
+										netRender->GetData(lineOfImageLowMem);
+										last = (int*)&lineOfImageLowMem[width];
+									}
 
-								image->ConvertTo8bit();
-								WriteLog("Image converted to 8-bit");
-								image->UpdatePreview();
-								WriteLog("Preview created");
-								if (outputDarea != NULL)
-								{
-									DrawHistogram();
-									DrawHistogram2();
-									WriteLog("Histograms refreshed");
-									image->RedrawInWidget(outputDarea);
-									WriteLog("Image redrawn in window");
+									int y = *last;
+									if(y<height)
+									{
+										for(int x=0; x<width; x++)
+										{
+											if(!image->IsLowMemMode())
+											{
+												image->GetComplexImagePtr()[x+y*width] = lineOfImage[x].complex;
+												image->PutPixelAlpha(x,y,lineOfImage[x].alpha);
+												image->PutPixelZBuffer(x,y,lineOfImage[x].zBuffer);
+												image->PutPixelColor(x,y,lineOfImage[x].colorIndexBuf16);
+											}
+											else
+											{
+												image->GetImage16Ptr()[x+y*width] = lineOfImageLowMem[x].image16;
+												image->PutPixelAlpha(x,y,lineOfImageLowMem[x].alpha);
+												image->PutPixelZBuffer(x,y,lineOfImageLowMem[x].zBuffer);
+												image->PutPixelColor(x,y,lineOfImageLowMem[x].colorIndexBuf16);
+											}
+										}
+										thread_done[y] = 99;
+										newLineRendered = true;
+									}
 								}
 							}
-
-							double refreshEndTime = real_clock();
-							//printf("RefreshTime: %f\n", refreshEndTime - refreshStartTime);
-							refresh_skip = (refreshEndTime - refreshStartTime) * 50;
-							refresh_index = 0;
-							WriteLogDouble("Image refreshed", refreshEndTime - refreshStartTime);
 						}
 					}
 
-					if (outputDarea != NULL)
+					if (netRender->IsClient())
 					{
-						//progress bar
-						char progressText[1000];
-						double progressiveDone, percent_done;
-						if (progressive == progressiveStart) progressiveDone = 0;
-						else progressiveDone = 0.25 / (progressive * progressive);
-						if (progressiveStart == 1)
+						//receiving list of already rendered lines
+						netRender->receiveDataFromServer(command);
+						if(!strcmp(command, "lst"))
 						{
-							percent_done = ((double) done / height) * 100.0;
+							netRender->GetData(thread_done_from_server);
+							memcpy(thread_done, thread_done_from_server, sizeof(int)*height);
 						}
-						else
+
+						//sending rendered line
+						if(!strcmp(command, "get"))
 						{
-							percent_done = (((double) done / height) * 3.0 / 4.0 / progressive + progressiveDone) * 100.0;
+							bool sentSomething = false;
+							for(int y=0; y<height; y++)
+							{
+								if(linesToSend[y] && thread_done_from_server[y] == 0)
+								{
+									linesToSend[y] = false;
+									for(int x=0; x<width; x++)
+									{
+										if(!image->IsLowMemMode())
+										{
+											lineOfImage[x].complex = image->GetComplexImagePtr()[x+y*width];
+											lineOfImage[x].alpha = image->GetPixelAlpha(x,y);
+											lineOfImage[x].colorIndexBuf16 = image->GetPixelColor(x,y);
+											lineOfImage[x].zBuffer = image->GetPixelZBuffer(x,y);
+										}
+										else
+										{
+											lineOfImageLowMem[x].image16 = image->GetImage16Ptr()[x+y*width];
+											lineOfImageLowMem[x].alpha = image->GetPixelAlpha(x,y);
+											lineOfImageLowMem[x].colorIndexBuf16 = image->GetPixelColor(x,y);
+											lineOfImageLowMem[x].zBuffer = image->GetPixelZBuffer(x,y);
+										}
+									}
+									int *last;
+									if(!image->IsLowMemMode())
+										last = (int*)&lineOfImage[width];
+									else
+										last = (int*)&lineOfImageLowMem[width];
+									*last = y;
+
+									if(!image->IsLowMemMode())
+										netRender->sendDataToServer(lineOfImage, sizeof(sAllImageData)*(width+1),"GET");
+									else
+										netRender->sendDataToServer(lineOfImageLowMem, sizeof(sAllImageDataLowMem)*(width+1),"GET");
+
+									sentSomething = true;
+									break;
+								}
+							}
+							if(!sentSomething)
+							{
+								//sending null answer when there si nothing to send
+								netRender->sendDataToServer(NULL,0,"GET");
+							}
+						}
+
+						if(!strcmp(command, "stp"))
+						{
+							programClosed = true;
+						}
+					}
+
+					if(!netRender->IsClient() && !netRender->IsServer()) g_usleep(100000);
+
+
+					if (!noGUI)
+					{
+						if (progressive < progressiveStart || netRender->IsServer() || netRender->IsClient())
+						{
+
+							if (real_clock() - refreshEndTime >= refresh_skip)
+							{
+								double refreshStartTime = real_clock();
+
+								if (image->IsPreview())
+								{
+									param.SSAOEnabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(Interface.checkSSAOEnabled) );
+									if (param.SSAOEnabled && !image->IsLowMemMode() && !netRender->IsClient() && !netRender->IsServer())
+									{
+										param.SSAOQuality = gtk_adjustment_get_value(GTK_ADJUSTMENT(Interface.adjustmentSSAOQuality) );
+										PostRendering_SSAO(image, param.doubles.persp, param.SSAOQuality / 2, param.perspectiveType, param.quiet);
+										WriteLog("SSAO rendered");
+									}
+									image->CompileImage();
+									WriteLog("Image compiled");
+
+									image->ConvertTo8bit();
+									WriteLog("Image converted to 8-bit");
+									image->UpdatePreview();
+									WriteLog("Preview created");
+									if (outputDarea != NULL)
+									{
+										DrawHistogram();
+										DrawHistogram2();
+										WriteLog("Histograms refreshed");
+										image->RedrawInWidget(outputDarea);
+										WriteLog("Image redrawn in window");
+									}
+								}
+
+								refreshEndTime = real_clock();
+								//printf("RefreshTime: %f\n", refreshEndTime - refreshStartTime);
+								refresh_skip = (refreshEndTime - refreshStartTime) * 10;
+								WriteLogDouble("Image refreshed", refreshEndTime - refreshStartTime);
+							}
+						}
+					}
+
+					if (!noGUI)
+					{
+						if (outputDarea != NULL)
+						{
+							//progress bar
+							char progressText[1000];
+							double progressiveDone, percent_done;
+							if (progressive == progressiveStart) progressiveDone = 0;
+							else progressiveDone = 0.25 / (progressive * progressive);
+							if (progressiveStart == 1)
+							{
+								percent_done = ((double) done2 / height) * 100.0;
+							}
+							else
+							{
+								percent_done = (((double) done2 / height) * 3.0 / 4.0 / progressive + progressiveDone) * 100.0;
+							}
+
+							if (param.noOfTiles > 1)
+							{
+								percent_done = (param.tileCount + percent_done / 100.0) / (param.noOfTiles * param.noOfTiles) * 100.0;
+							}
+							if (Interface_data.animMode)
+							{
+								percent_done = (param.fractal.frameNo - param.startFrame + percent_done / 100.0) / (param.endFrame - param.startFrame + 1) * 100.0;
+							}
+
+							double time = real_clock() - start_time;
+							double time_to_go = (100.0 - percent_done) * time / percent_done;
+							int togo_time_s = (int) time_to_go % 60;
+							int togo_time_min = (int) (time_to_go / 60) % 60;
+							int togo_time_h = time_to_go / 3600;
+							int time_s = (int) time % 60;
+							int time_min = (int) (time / 60) % 60;
+							int time_h = time / 3600;
+							double iterations_per_sec = N_counter / time;
+							//double avgDEerror = DEerror / DE_counterForDEerror*100.0;
+							double avgMissedDE = (double) Missed_DE_counter / Pixel_counter * 100.0;
+							sprintf(progressText, "%.3f%%, to go %dh%dm%ds, elapsed %dh%dm%ds, iters/s %.0f, Missed DE %.3f%%", percent_done, togo_time_h, togo_time_min, togo_time_s, time_h,
+									time_min, time_s, iterations_per_sec, avgMissedDE);
+							gtk_progress_bar_set_text(GTK_PROGRESS_BAR(Interface.progressBar), progressText);
+							gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(Interface.progressBar), percent_done / 100.0);
 						}
-						double time = real_clock() - start_time;
-						double time_to_go = (100.0 - percent_done) * time / percent_done;
-						int togo_time_s = (int) time_to_go % 60;
-						int togo_time_min = (int) (time_to_go / 60) % 60;
-						int togo_time_h = time_to_go / 3600;
-						int time_s = (int) time % 60;
-						int time_min = (int) (time / 60) % 60;
-						int time_h = time / 3600;
-						double iterations_per_sec = N_counter / time;
-						//double avgDEerror = DEerror / DE_counterForDEerror*100.0;
-						double avgMissedDE = (double) Missed_DE_counter / Pixel_counter * 100.0;
-						sprintf(progressText, "%.3f%%, to go %dh%dm%ds, elapsed %dh%dm%ds, iters/s %.0f, Missed DE %.3f%%", percent_done, togo_time_h, togo_time_min, togo_time_s, time_h,
-								time_min, time_s, iterations_per_sec, avgMissedDE);
-						gtk_progress_bar_set_text(GTK_PROGRESS_BAR(Interface.progressBar), progressText);
-						gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(Interface.progressBar), percent_done / 100.0);
 					}
 					//refreshing GUI
-					if (image->IsPreview())
+					if (image->IsPreview() || netRender->IsClient())
 					{
 						while (gtk_events_pending())
 							gtk_main_iteration();
 					}
+
+					done2 = 0;
+					for(int i=0; i<height; i++)
+					{
+						if(thread_done[i] > 0)
+						{
+							done2++;
+						}
+					}
 				}
 			}
 			//when rendered all of image lines wait for finishing threads
@@ -1179,6 +1538,16 @@ void Render(sParamRender param, cImage *image, GtkWidget *outputDarea)
 				WriteLogDouble("Thread finish confirmed", i);
 				//printf("Rendering thread #%d finished\n", i + 1);
 			}
+
+			//terminate rendering by all clients
+			if (netRender->IsServer())
+			{
+				for (int i = 0; i < netRender->getNoOfClients(); i++)
+				{
+					netRender->sendDataToClient(NULL, 0, "stp", i, param.fractal.frameNo);
+				}
+			}
+
 			if (programClosed) break;
 		} //next progress
 		printf("\n");
@@ -1187,14 +1556,14 @@ void Render(sParamRender param, cImage *image, GtkWidget *outputDarea)
 
 		//*** postprocessing
 
-		if (param.SSAOEnabled && !programClosed)
+		if (param.SSAOEnabled && !programClosed && !netRender->IsClient())
 		{
 			PostRendering_SSAO(image, param.doubles.persp, param.SSAOQuality, param.perspectiveType, param.quiet);
 			WriteLog("SSAO rendered");
 		}
 		image->CompileImage();
 		WriteLog("Image compiled");
-		if (param.DOFEnabled && !programClosed)
+		if (param.DOFEnabled && !programClosed && !netRender->IsClient())
 		{
 			double DOF_focus = pow(10, param.doubles.DOFFocus / 10.0 - 2.0) - 1.0 / param.doubles.persp;
 			PostRendering_DOF(image, param.doubles.DOFRadius * width / 1000.0, DOF_focus, param.doubles.persp);
@@ -1248,6 +1617,12 @@ void Render(sParamRender param, cImage *image, GtkWidget *outputDarea)
 			}
 		}
 		delete[] thread_done;
+		delete[] thread_param;
+		delete[] Thread;
+		delete[] err;
+		delete[] linesToSend;
+		if (lineOfImage) delete[] lineOfImage;
+		if (lineOfImageLowMem) delete[] lineOfImageLowMem;
 	}
 	else
 	{
@@ -1337,6 +1712,8 @@ int get_cpu_count()
 
 	GetSystemInfo(&info);
 	ret = info.dwNumberOfProcessors;
+#elif defined(__sgi)
+	ret = (int) sysconf(_SC_NPROC_ONLN);
 #else               /*other unix - try sysconf*/
 	ret = (int) sysconf(_SC_NPROCESSORS_ONLN);
 #endif  /* WINDOWS */
@@ -1545,6 +1922,9 @@ int main(int argc, char *argv[])
 		}
 	}
 
+	//initialise netRender
+	netRender = new CNetRender(MANDELBULBER_VERSION * 1000, NR_THREADS);
+
 	//set program home directory
 	if (!chdir(data_directory))
 	{
@@ -1581,24 +1961,33 @@ int main(int argc, char *argv[])
 		}
 		else
 		{
-			if (!noGUIsettingsLoaded)
+		  g_type_init();
+			if(!noGUIdata.netrenderMode)
 			{
-				char settingsFile2[1000];
-				sprintf(settingsFile2, "settings/%s", noGUIdata.settingsFile);
-				if (!LoadSettings(settingsFile2, noGUIdata.fractparams))
+				if (!noGUIsettingsLoaded)
 				{
-					printf("ERROR! Can't open settings file: %s\n", noGUIdata.settingsFile);
-					return -1;
+					char settingsFile2[1000];
+					sprintf(settingsFile2, "settings/%s", noGUIdata.settingsFile);
+					if (!LoadSettings(settingsFile2, noGUIdata.fractparams))
+					{
+						printf("ERROR! Can't open settings file: %s\n", noGUIdata.settingsFile);
+						return -1;
+					}
 				}
+				WriteLog("Default settings loaded");
+				Interface_data.animMode = noGUIdata.animMode;
+				Interface_data.playMode = noGUIdata.playMode;
+				Interface_data.recordMode = false;
+				Interface_data.continueRecord = false;
+				Interface_data.keyframeMode = noGUIdata.keyframeMode;
+				MainRender();
+				WriteLog("Rendering completely finished");
+			}
+			else
+			{
+				printf("Starting net client application\n");
+				NoGUIClientEnable();
 			}
-			WriteLog("Default settings loaded");
-			Interface_data.animMode = noGUIdata.animMode;
-			Interface_data.playMode = noGUIdata.playMode;
-			Interface_data.recordMode = false;
-			Interface_data.continueRecord = false;
-			Interface_data.keyframeMode = noGUIdata.keyframeMode;
-			MainRender();
-			WriteLog("Rendering completely finished");
 		}
 
 		WriteLog("Memory for parameters released");
@@ -1937,6 +2326,8 @@ void MainRender(void)
 
 	int tiles = fractParam.noOfTiles;
 
+	start_time = real_clock();
+
 	printf("************ Rendering frames *************\n");
 	WriteLog("************ Rendering frames *************");
 	for (int index = startFrame; index < endFrame; index++)
@@ -2202,7 +2593,7 @@ void MainRender(void)
 							else if (Interface_data.imageFormat == imgFormatPNG16)
 							{
 								filename2 = IndexFilename(fractParam.file_destination, "png", index2);
-								SavePNG16(filename2.c_str(), 100, width, height, &mainImage);
+								SavePNG16(filename2.c_str(), 100, width, height, mainImage.GetImage16Ptr());
 							}
 							else if (Interface_data.imageFormat == imgFormatPNG16Alpha)
 							{
