@@ -24,6 +24,7 @@
 #include "undo.hpp"
 #include "timeline.hpp"
 #include "cl_support.hpp"
+#include "netrender.hpp"
 
 using namespace std;
 
@@ -42,7 +43,7 @@ gboolean motion_notify_event(GtkWidget *widget, GdkEventMotion *event)
 	return TRUE;
 }
 
-gboolean CallerTimerLoop(GtkWidget *widget)
+gboolean CallerTimerLoop(void)
 {
 	if (isRendering && renderRequest && !Interface_data.animMode)
 	{
@@ -58,6 +59,7 @@ gboolean CallerTimerLoop(GtkWidget *widget)
 		Interface_data.recordMode = false;
 		Interface_data.continueRecord = false;
 		gdk_threads_enter();
+		printf("Starting\n");
 		MainRender();
 		gdk_threads_leave();
 	}
@@ -145,8 +147,11 @@ gboolean pressed_button_on_image(GtkWidget *widget, GdkEventButton *event)
 				double aspectRatio = (double) width / height;
 
 				enumPerspectiveType perspectiveType = (enumPerspectiveType)gtk_combo_box_get_active(GTK_COMBO_BOX(Interface.comboPerspectiveType));
+				if(perspectiveType == fishEyeCut) perspectiveType = fishEye;
+
 				if (perspectiveType == fishEye || perspectiveType == equirectangular)
 				{
+					y /= 1e12;
 					x2 = M_PI * ((double) x / width - 0.5) * aspectRatio;
 					z2 = M_PI * ((double) z / height - 0.5);
 				}
@@ -431,6 +436,14 @@ void StopRendering(GtkWidget *widget, gpointer data)
 	programClosed = true;
 	isPostRendering = false;
 	printf("Rendering terminated\n");
+
+	if (netRender->IsServer())
+	{
+		for (int i = 0; i < netRender->getNoOfClients(); i++)
+		{
+			netRender->sendDataToClient(NULL, 0, "stp", i, 0);
+		}
+	}
 }
 
 void PressedApplyBrigtness(GtkWidget *widget, gpointer data)
@@ -570,7 +583,7 @@ void PressedSaveImagePNG16(GtkWidget *widget, gpointer data)
 	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
 	{
 		const char *filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
-		SavePNG16(filename, 100, mainImage.GetWidth(), mainImage.GetHeight(), &mainImage);
+		SavePNG16(filename, 100, mainImage.GetWidth(), mainImage.GetHeight(), mainImage.GetImage16Ptr());
 		strcpy(lastFilenameImage, filename);
 	}
 	gtk_widget_destroy(dialog);
@@ -1274,18 +1287,18 @@ void RecalculateIFSParams(sFractal &fractal)
 
 void CreateFormulaSequence(sFractal &fractal)
 {
-	fractal.formulaSequence.resize(fractal.N);
-	fractal.hybridPowerSequence.resize(fractal.N);
+	fractal.formulaSequence.resize(fractal.doubles.N);
+	fractal.hybridPowerSequence.resize(fractal.doubles.N);
 
 	int number = 0;
-	while (number < fractal.N)
+	while (number < (int)fractal.doubles.N)
 	{
 		for (int hybrid_n = 0; hybrid_n < HYBRID_COUNT - 1; ++hybrid_n)
 		{
 			if (fractal.hybridFormula[hybrid_n] == none) continue;
 			for (int i = 0; i < fractal.hybridIters[hybrid_n]; ++i)
 			{
-				if (number < fractal.N)
+				if (number < fractal.doubles.N)
 				{
 					fractal.formulaSequence[number] = fractal.hybridFormula[hybrid_n];
 					fractal.hybridPowerSequence[number] = fractal.doubles.hybridPower[hybrid_n];
@@ -1294,12 +1307,12 @@ void CreateFormulaSequence(sFractal &fractal)
 			}
 		}
 
-		int temp_end = fractal.N;
+		int temp_end = (int)fractal.doubles.N;
 		if (fractal.hybridCyclic) temp_end = fractal.hybridIters[HYBRID_COUNT - 1];
 
 		if (fractal.hybridFormula[HYBRID_COUNT - 1] != none) for (int i = 0; i < temp_end; i++)
 		{
-			if (number < fractal.N)
+			if (number < (int)fractal.doubles.N)
 			{
 				fractal.formulaSequence[number] = fractal.hybridFormula[HYBRID_COUNT - 1];
 				fractal.hybridPowerSequence[number] = fractal.doubles.hybridPower[HYBRID_COUNT - 1];
@@ -1913,7 +1926,7 @@ void PressedIFSDefaultDodeca(GtkWidget *widget, gpointer widget_pointer)
 	ReadInterface(&params);
 	undoBuffer.SaveUndo(&params);
 
-	double phi = (1 + sqrt(5))/2.0;
+	double phi = (1 + sqrt(5.0))/2.0;
 	params.fractal.IFS.doubles.direction[0].x = phi*phi;
 	params.fractal.IFS.doubles.direction[0].y = 1.0;
 	params.fractal.IFS.doubles.direction[0].z = -phi;
@@ -1951,7 +1964,7 @@ void PressedIFSDefaultIcosa(GtkWidget *widget, gpointer widget_pointer)
 	ReadInterface(&params);
 	undoBuffer.SaveUndo(&params);
 
-	double phi = (1 + sqrt(5))/2.0;
+	double phi = (1 + sqrt(5.0))/2.0;
 	params.fractal.IFS.doubles.direction[3].x = -phi*phi;
 	params.fractal.IFS.doubles.direction[3].y = 1.0;
 	params.fractal.IFS.doubles.direction[3].z = phi;
@@ -2147,7 +2160,7 @@ void AutoDEStep(bool highQuality)
 	for(int i=0; i<5; i++)
 		fractParams.volumetricLightEnabled[i] = false;
 	fractParams.imageSwitches.raytracedReflections = false;
-	if (fractParams.fractal.N < 200) fractParams.fractal.N = 200;
+	if (fractParams.fractal.doubles.N < 200) fractParams.fractal.doubles.N = 200;
 	if (fractParams.fractal.limits_enabled) return;
 
 	int scanCount = 0;
@@ -2279,13 +2292,15 @@ void PressedCopyToClipboard(GtkWidget *widget, gpointer data)
 		if (result != lSize)
 		{
 			printf("Reading error of clipboard temporary file");
+			fclose(pFile);
+			delete [] buffer;
 			return;
 		}
 
 		gtk_clipboard_set_text(clipboard, buffer, lSize);
 
 		fclose(pFile);
-		delete buffer;
+		delete [] buffer;
 		remove("settings/.clipboard");
 	}
 }
@@ -2420,3 +2435,242 @@ void ChangedIterFogEnable(GtkWidget *widget, gpointer data)
 	gtk_widget_set_sensitive(Interface.edit_iterFogOpacity, active);
 	gtk_widget_set_sensitive(Interface.edit_iterFogOpacityTrim, active);
 }
+
+void PressedSaveAllImageLayers(GtkWidget *widget, gpointer data)
+{
+	GtkWidget *dialog;
+	dialog = gtk_file_chooser_dialog_new("Save all layers image as...", GTK_WINDOW(window_interface), GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE,
+			GTK_RESPONSE_ACCEPT, NULL);
+	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(dialog), TRUE);
+
+	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dialog), lastFilenameImage);
+
+	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
+	{
+		const char *filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
+		SaveAllImageLayers(filename, &mainImage);
+		strcpy(lastFilenameImage, filename);
+	}
+	gtk_widget_destroy(dialog);
+}
+
+void PressedServerEnable(GtkWidget *widget, gpointer data)
+{
+	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(Interface.checkNetRenderServerEnable)))
+	{
+		char status[1000];
+		bool result = netRender->SetServer((char*)gtk_entry_get_text(GTK_ENTRY(Interface.edit_netRenderServerPort)), status);
+		gtk_label_set_text(GTK_LABEL(Interface.label_serverStatus),status);
+		if(result)
+		{
+			gtk_widget_set_sensitive(Interface.checkNetRenderServerScan, true);
+		}
+	}
+	else
+	{
+		netRender->DeleteServer();
+		gtk_label_set_text(GTK_LABEL(Interface.label_serverStatus),"status: not enabled");
+		gtk_widget_set_sensitive(Interface.checkNetRenderServerScan, false);
+	}
+}
+
+void PressedClientEnable(GtkWidget *widget, gpointer data)
+{
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(Interface.checkNetRenderClientEnable) ))
+	{
+		char status[1000];
+		bool result = netRender->SetClient((char*) gtk_entry_get_text(GTK_ENTRY(Interface.edit_netRenderClientPort) ),
+				(char*) gtk_entry_get_text(GTK_ENTRY(Interface.edit_netRenderClientName) ), status);
+		gtk_label_set_text(GTK_LABEL(Interface.label_clientStatus), status);
+		if (result)
+		{
+			while (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(Interface.checkNetRenderClientEnable) ))
+			{
+				while (gtk_events_pending())
+					gtk_main_iteration();
+
+				if(!isRendering && !renderRequest)
+				{
+					char command[4];
+					size_t size = netRender->receiveDataFromServer(command);
+
+					if (!strcmp(command, "set"))
+					{
+						char *buffer = new char[size];
+						netRender->GetData(buffer);
+						GetSettingsfromServer(buffer, size);
+						while (gtk_events_pending())
+							gtk_main_iteration();
+						delete buffer;
+					}
+					if (!strcmp(command, "run"))
+					{
+						Interface_data.animMode = false;
+						Interface_data.playMode = false;
+						Interface_data.recordMode = false;
+						Interface_data.continueRecord = false;
+						Interface_data.keyframeMode = false;
+
+						programClosed = true;
+						isPostRendering = false;
+						renderRequest = true;
+					}
+				}
+			}
+		}
+	}
+	else
+	{
+		netRender->DeleteClient();
+		gtk_label_set_text(GTK_LABEL(Interface.label_clientStatus),"status: not enabled");
+	}
+}
+
+void NoGUIClientEnable(void)
+{
+	char status[1000];
+
+	bool result = false;
+	while(!result)
+	{
+		g_usleep(1000000);
+		result = netRender->SetClient(noGUIdata.netRenderPortString, noGUIdata.netRenderIPString, status);
+		printf("NetRender status: %s\n", status);
+	}
+
+	if (result)
+	{
+		g_timeout_add (100,(GSourceFunc)CallerTimerLoop,NULL);
+
+		while (true)
+		{
+			while (gtk_events_pending())
+				gtk_main_iteration();
+
+			if (!isRendering && !renderRequest)
+			{
+				char command[4];
+				size_t size = netRender->receiveDataFromServer(command);
+
+				if (!strcmp(command, "set"))
+				{
+					char *buffer = new char[size];
+					netRender->GetData(buffer);
+					GetSettingsfromServer(buffer, size);
+					delete buffer;
+				}
+				if (!strcmp(command, "run"))
+				{
+					Interface_data.animMode = false;
+					Interface_data.playMode = false;
+					Interface_data.recordMode = false;
+					Interface_data.continueRecord = false;
+					Interface_data.keyframeMode = false;
+
+					programClosed = true;
+					isPostRendering = false;
+					renderRequest = true;
+				}
+			}
+		}
+	}
+}
+
+void PressedServerScan(GtkWidget *widget, gpointer data)
+{
+	char status[1000];
+
+	if (netRender->IsServer())
+	{
+		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(Interface.checkNetRenderServerScan)))
+		{
+			gtk_label_set_text(GTK_LABEL(Interface.label_serverStatus), "status: waiting for clients");
+
+			while (gtk_events_pending())
+				gtk_main_iteration();
+
+			while (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(Interface.checkNetRenderServerScan)))
+			{
+				bool result = netRender->WaitForClient(status);
+				if (result)
+				{
+					gtk_label_set_text(GTK_LABEL(Interface.label_serverStatus), status);
+				}
+				while (gtk_events_pending())
+					gtk_main_iteration();
+			}
+		}
+		else
+		{
+			sprintf(status, "status: %d clients connected", netRender->getNoOfClients());
+			gtk_label_set_text(GTK_LABEL(Interface.label_serverStatus), status);
+		}
+	}
+}
+
+bool SendSettingsToClients(sParamRender fractParamToSend)
+{
+	printf("Sending settings to clients\n");
+	SaveSettings("settings/.temp_send", fractParamToSend, true);
+
+	FILE * pFile;
+	pFile = fopen("settings/.temp_send", "rb");
+	if (pFile != NULL)
+	{
+		// obtain file size:
+		fseek(pFile, 0, SEEK_END);
+		unsigned int lSize = ftell(pFile);
+		rewind(pFile);
+
+		char *buffer = new char[lSize];
+
+		// copy the file into the buffer:
+		size_t result = fread(buffer, 1, lSize, pFile);
+		if (result != lSize)
+		{
+			printf("Reading error of clipboard temporary file");
+			fclose(pFile);
+			delete [] buffer;
+			return false;
+		}
+
+		for(int i=0; i<netRender->getNoOfClients(); i++)
+		{
+			netRender->sendDataToClient(buffer, lSize, (char*)"set", i, 0);
+		}
+
+		fclose(pFile);
+		delete [] buffer;
+		remove("settings/.temp_send");
+
+		printf("Settings sent\n");
+	}
+	return true;
+}
+
+bool GetSettingsfromServer(char *data, size_t size)
+{
+	FILE * pFile;
+	pFile = fopen("settings/.temp_rcv", "wb");
+	if (pFile != NULL)
+	{
+		fwrite(data, 1, size, pFile);
+		fclose(pFile);
+
+		sParamRender fractParamLoaded;
+
+		if(noGUI)
+		{
+			LoadSettings("settings/.temp_rcv", noGUIdata.fractparams);
+			Params2InterfaceData(&noGUIdata.fractparams);
+		}
+		else
+		{
+			LoadSettings("settings/.temp_rcv", fractParamLoaded);
+			Params2InterfaceData(&fractParamLoaded);
+			WriteInterface(&fractParamLoaded);
+		}
+		//remove("settings/.temp_rcv");
+	}
+	return true;
+}
