@@ -34,6 +34,7 @@ struct sRGB
 
 struct sComplexImage
 {
+	sRGB16 backgroundBuf16;
 	unsigned short shadowsBuf16;
 	unsigned short shadingBuf16;
 	unsigned short specularBuf16;
@@ -42,7 +43,6 @@ struct sComplexImage
 	unsigned short glowBuf16;
 	sRGB16 reflectBuf16;
 	sRGB16 ambientBuf16;
-	sRGB16 backgroundBuf16;
 	sRGB16 volumetricLight;
 	unsigned short fogDensity16;
 };
@@ -83,6 +83,22 @@ struct sImageSwitches
 	bool iterFogEnabled;
 };
 
+struct sAllImageData
+{
+	unsigned short alpha;
+	float zBuffer;
+	unsigned short colorIndexBuf16;
+	sComplexImage complex;
+};
+
+struct sAllImageDataLowMem
+{
+	unsigned short alpha;
+	float zBuffer;
+	unsigned short colorIndexBuf16;
+	sRGB16 image16;
+};
+
 class cImage
 {
 public:
@@ -112,6 +128,11 @@ public:
   inline sRGB16 GetPixelAmbient(int x, int y)  {if (x >= 0 && x < width && y >= 0 && y < height) return complexImage[x + y * width].ambientBuf16; else return Black16();}
   inline float GetPixelZBuffer(int x, int y)  {if (x >= 0 && x < width && y >= 0 && y < height) return zBuffer[x + y * width]; else return 1e20;}
   sRGB16* GetImage16Ptr(void) {return image16;}
+  sComplexImage* GetComplexImagePtr(void) {return complexImage;}
+  unsigned short* GetColourIndexBufPtr(void) {return colorIndexBuf16;}
+  unsigned short* GetAlphaBufPtr(void) {return alpha;}
+  float* GetZBufferPtr(void) {return zBuffer;}
+
   void CompileImage(void);
   void SetPalette(sRGB *palette);
   sRGB* GetPalettePtr() {return palette;}
