@@ -16,6 +16,7 @@
 #include "Render3D.h"
 #include "interface.h"
 #include "primitives.h"
+#include <stdlib.h>
 
 /**
  * Compute the fractal at the point, in one of the various modes
@@ -26,6 +27,21 @@
  *		 delta_DE1, delta_DE2: Returns radius
  */
 
+unsigned int MixNumbers(double a, double b, double c)
+{
+	unsigned int result = int(a*12356312.0)^int(b*23564234.0)^int(c*35564353.0);
+	//printf("%ud\n", result);
+	return result;
+}
+
+int Noise(int seed)
+{
+	int x = seed;
+	x = (x<<7) ^ x;
+	return abs((x * (x * x * 15731 + 789221) + 1376312589) & 0x7fffffff);
+}
+
+
 template<int Mode>
 double Compute(CVector3 z, const sFractal &par, int *iter_count)
 {
@@ -41,6 +57,7 @@ double Compute(CVector3 z, const sFractal &par, int *iter_count)
 	double ph_dz = 0;
 	double th_dz = 0;
 	double p = par.doubles.power; //mandelbulb power
+	int N = par.doubles.N;
 
 	CVector3 constant;
 
@@ -75,7 +92,7 @@ double Compute(CVector3 z, const sFractal &par, int *iter_count)
 	double r = z.Length();
 
 	double min = 1e200;
-	for (L = 0; L < par.N; L++)
+	for (L = 0; L < N; L++)
 	{
 		if (hybridEnabled)
 		{
@@ -824,6 +841,7 @@ double Compute(CVector3 z, const sFractal &par, int *iter_count)
 			}
 			case kaleidoscopic:
 			{
+
 				if (par.IFS.absX) z.x = fabs(z.x);
 				if (par.IFS.absY) z.y = fabs(z.y);
 				if (par.IFS.absZ) z.z = fabs(z.z);
@@ -1103,6 +1121,18 @@ double Compute(CVector3 z, const sFractal &par, int *iter_count)
 				break;
 		}
 
+		if (Mode == orbitTrap)
+		{
+			CVector3 delta = z - par.doubles.fakeLightsOrbitTrap;
+			distance = delta.Length();
+			if (L >= par.fakeLightsMinIter && L <= par.fakeLightsMaxIter && distance < min) min = distance;
+			if (distance > 1000)
+			{
+				distance = min;
+				break;
+			}
+		}
+
 		if (actualFormula == menger_sponge || actualFormula == kaleidoscopic)
 		{
 			if (r > 1000)
@@ -1140,12 +1170,24 @@ double Compute(CVector3 z, const sFractal &par, int *iter_count)
 			}
 			else if (Mode == colouring) //mode 1
 			{
-				distance = z.Length();
-				if (distance < min) min = distance;
-				if (distance > 1e15)
+				if(par.primitives.onlyPlane)
 				{
-					distance = min;
-					break;
+					distance = z.Length();
+					if (distance > 1e15)
+					{
+						distance = (L - log(log(r) / log(N)) / log(p))/100.0;
+						break;
+					}
+				}
+				else
+				{
+					distance = z.Length();
+					if (distance < min) min = distance;
+					if (distance > 1e15)
+					{
+						distance = min;
+						break;
+					}
 				}
 			}
 		}
@@ -1165,7 +1207,7 @@ double Compute(CVector3 z, const sFractal &par, int *iter_count)
 
 	if (Mode == normal)
 	{
-		if (L == par.N)
+		if (L == N)
 			distance = 0;
 		return distance;
 	}
@@ -1176,6 +1218,9 @@ double Compute(CVector3 z, const sFractal &par, int *iter_count)
 	if (Mode == fake_AO)
 		return distance;
 
+	if (Mode == orbitTrap)
+		return distance;
+
 	if (Mode == colouring)
 	{
 		if (par.formula == hybrid)
@@ -1255,96 +1300,96 @@ double CalculateDistance(CVector3 point, sFractal &params, bool *max_iter)
 		}
 	}
 
-	if (params.analitycDE)
+	if(!params.primitives.onlyPlane)
 	{
-		distance = Compute<normal>(point, params, &L);
-		if (max_iter != NULL)
+		if (params.analitycDE)
 		{
-			if (L == params.N)
-				*max_iter = true;
-			else
-				*max_iter = false;
-		}
-		params.itersOut = L;
-
-		if (L < params.minN && distance < params.doubles.detailSize)
-			distance = params.doubles.detailSize;
-		
-		if (params.interiorMode)
-		{
-			if (distance < 0.5 * params.doubles.detailSize || L == params.N)
+			distance = Compute<normal>(point, params, &L);
+			if (max_iter != NULL)
 			{
-				distance = params.doubles.detailSize;
-				if (max_iter != NULL)
-					*max_iter = false;
+				if (L == (int)params.doubles.N) *max_iter = true;
+				else *max_iter = false;
+			}
+			params.itersOut = L;
+
+			if (L < params.minN && distance < params.doubles.detailSize) distance = params.doubles.detailSize;
+
+			if (params.interiorMode)
+			{
+				if (distance < 0.5 * params.doubles.detailSize || L == (int)params.doubles.N)
+				{
+					distance = params.doubles.detailSize;
+					if (max_iter != NULL) *max_iter = false;
+				}
+			}
+		}
+		else
+		{
+			double deltaDE = 1e-10;
+
+			double r = Compute<deltaDE1>(point, params, &L);
+			int retval = L;
+			params.itersOut = L;
+
+			point.x += deltaDE;
+			point.y += 0;
+			point.z += 0;
+			double r2 = Compute<deltaDE2>(point, params, &L);
+			double dr1 = fabs(r2 - r) / deltaDE;
+
+			point.x -= deltaDE;
+			point.y += deltaDE;
+			point.z += 0;
+			r2 = Compute<deltaDE2>(point, params, &L);
+			double dr2 = fabs(r2 - r) / deltaDE;
+
+			point.x += 0;
+			point.y -= deltaDE;
+			point.z += deltaDE;
+			r2 = Compute<deltaDE2>(point, params, &L);
+			double dr3 = fabs(r2 - r) / deltaDE;
+
+			double dr = sqrt(dr1 * dr1 + dr2 * dr2 + dr3 * dr3);
+
+			if (params.linearDEmode)
+			{
+				distance = 0.5 * r / dr;
+			}
+			else
+			{
+				distance = 0.5 * r * log(r) / dr;
+			}
+
+			if (retval == (int)params.doubles.N)
+			{
+				if (max_iter != NULL) *max_iter = true;
+				distance = 0;
+			}
+			else if (max_iter != NULL) *max_iter = false;
+
+			if (L < params.minN && distance < params.doubles.detailSize) distance = params.doubles.detailSize;
+
+			if (params.interiorMode)
+			{
+				if (distance < 0.5 * params.doubles.detailSize || retval == 256)
+				{
+					distance = params.doubles.detailSize;
+					if (max_iter != NULL) *max_iter = false;
+				}
 			}
 		}
 	}
 	else
 	{
-		double deltaDE = 1e-10;
-
-		double r = Compute<deltaDE1>(point, params, &L);
-		int retval = L;
-		params.itersOut = L;
-
-		point.x += deltaDE;
-		point.y += 0;
-		point.z += 0;
-		double r2 = Compute<deltaDE2>(point, params, &L);
-		double dr1 = fabs(r2 - r) / deltaDE;
-
-		point.x -= deltaDE;
-		point.y += deltaDE;
-		point.z += 0;
-		r2 = Compute<deltaDE2>(point, params, &L);
-		double dr2 = fabs(r2 - r) / deltaDE;
-
-		point.x += 0;
-		point.y -= deltaDE;
-		point.z += deltaDE;
-		r2 = Compute<deltaDE2>(point, params, &L);
-		double dr3 = fabs(r2 - r) / deltaDE;
-
-		double dr = sqrt(dr1 * dr1 + dr2 * dr2 + dr3 * dr3);
-
-		if(params.linearDEmode)
-		{
-			distance = 0.5 * r / dr;
-		}
-		else
-		{
-			distance = 0.5 * r * log(r) / dr;
-		}
-
-		if (retval == params.N)
-		{
-			if (max_iter != NULL)
-				*max_iter = true;
-			distance = 0;
-		}
-		else if (max_iter != NULL)
-			*max_iter = false;
-
-		if (L < params.minN && distance < params.doubles.detailSize)
-			distance = params.doubles.detailSize;
-
-		if (params.interiorMode)
-		{
-			if (distance < 0.5 * params.doubles.detailSize || retval == 256)
-			{
-				distance = params.doubles.detailSize;
-				if (max_iter != NULL)
-					*max_iter = false;
-			}
-		}
+		distance = 10.0;
+		if (max_iter != NULL) *max_iter = false;
 	}
 
 	//plane
 	if (params.primitives.planeEnable)
 	{
 		double planeDistance = PrimitivePlane(point, params.doubles.primitives.planeCentre, params.doubles.primitives.planeNormal);
-		if(planeDistance < distance) 	params.specialColour = 253;
+		if(!params.primitives.onlyPlane && planeDistance < distance) 	params.specialColour = 253;
 		distance = (planeDistance < distance) ? planeDistance : distance;
 
 	}
@@ -1401,3 +1446,4 @@ template double Compute<colouring>(CVector3, const sFractal&, int*);
 template double Compute<fake_AO>(CVector3, const sFractal&, int*);
 template double Compute<deltaDE1>(CVector3, const sFractal&, int*);
 template double Compute<deltaDE2>(CVector3, const sFractal&, int*);
+template double Compute<orbitTrap>(CVector3, const sFractal&, int*);
