@@ -57,6 +57,9 @@ struct sParamRenderD
 	double fastAoTune;
 	double iterFogOpacity;
 	double iterFogOpacityTrim;
+	double fakeLightsIntensity;
+	double fakeLightsVisibility;
+	double fakeLightsVisibilitySize;
 
 	sImageAdjustments imageAdjustments;
 };
@@ -91,6 +94,7 @@ struct sParamRender
 	bool fastGlobalIllumination; //enable fake global ilumination
 	bool slowShading; //enable fake gradient calculation for shading
 	bool textured_background; //enable testured background
+	bool background_as_fulldome;
 	bool recordMode; //path recording mode
 	bool continueRecord; //continue recording mode
 	bool playMode; //play mode
@@ -105,6 +109,8 @@ struct sParamRender
 	bool penetratingLights;
 	bool stereoEnabled;
 	bool quiet;
+	bool fishEyeCut;
+	bool fakeLightsEnabled;
 	sImageSwitches imageSwitches;
 
 	sRGB background_color1; //background colour
