@@ -8,11 +8,12 @@
 #ifndef INTERFACE_H_
 #define INTERFACE_H_
 
+#include <gtk/gtk.h>
 #include "fractparams.h"
 #include "cl_support.hpp"
 
-#define MANDELBULBER_VERSION 1.11
-#define MANDELBULBER_VERSION_STR "1.11"
+#define MANDELBULBER_VERSION 1.15
+#define MANDELBULBER_VERSION_STR "1.15"
 
 #ifndef SHARED_DIR
 	#ifndef WIN32
@@ -83,6 +84,9 @@ struct sInterface
 	GtkWidget *tab_label_primitiveBox;
 	GtkWidget *tab_label_primitiveSphere;
 	GtkWidget *tab_label_openCL;
+	GtkWidget *tabsNetRender;
+	GtkWidget *tab_label_server;
+	GtkWidget *tab_label_client;
 
 	GtkWidget *tab_box_view;
 	GtkWidget *tab_box_fractal;
@@ -100,6 +104,8 @@ struct sInterface
 	GtkWidget *tab_box_primitiveBox;
 	GtkWidget *tab_box_primitiveSphere;
 	GtkWidget *tab_box_openCL;
+	GtkWidget *tab_box_server;
+	GtkWidget *tab_box_client;
 
 	GtkWidget *boxMain;
 	GtkWidget *boxButtons;
@@ -230,6 +236,13 @@ struct sInterface
 	GtkWidget *boxOpenClEngineSettingsV;
 	GtkWidget *boxOpenClEngineSettingsH1;
 	GtkWidget *boxOpenClEngineSettingsH2;
+	GtkWidget *boxNetRenderServerV;
+	GtkWidget *boxNetRenderClientV;
+	GtkWidget *boxNetRenderServerH1;
+	GtkWidget *boxNetRenderClientH1;
+	GtkWidget *boxFakeLightsV;
+	GtkWidget *boxFakeLightsH1;
+	GtkWidget *boxFakeLightsH2;
 
 	GtkWidget *tableLimits;
 	GtkWidget *tableArrows;
@@ -286,6 +299,8 @@ struct sInterface
 	GtkWidget *frOpenClSettings;
 	GtkWidget *frOpenClInformation;
 	GtkWidget *frOpenClEngineSettings;
+	GtkWidget *frNetRender;
+	GtkWidget *frFakeLights;
 
 	GtkWidget *hSeparator1;
 	GtkWidget *hSeparator2;
@@ -365,6 +380,7 @@ struct sInterface
 	GtkWidget *buColorPrimitiveWater;
 	GtkWidget *buAutoFog;
 	GtkWidget *buMeasureActivation;
+	GtkWidget *buSaveAllImageLayers;
 
 	GtkWidget *edit_va;
 	GtkWidget *edit_vb;
@@ -545,6 +561,17 @@ struct sInterface
 	GtkWidget *edit_OpenCLPixelsPerJob;
 	GtkWidget *edit_iterFogOpacityTrim;
 	GtkWidget *edit_iterFogOpacity;
+	GtkWidget *edit_netRenderServerPort;
+	GtkWidget *edit_netRenderClientPort;
+	GtkWidget *edit_netRenderClientName;
+	GtkWidget *edit_fakeLightsOrbitTrapX;
+	GtkWidget *edit_fakeLightsOrbitTrapY;
+	GtkWidget *edit_fakeLightsOrbitTrapZ;
+	GtkWidget *edit_fakeLightsIntensity;
+	GtkWidget *edit_fakeLightsVisibility;
+	GtkWidget *edit_fakeLightsVisibilitySize;
+	GtkWidget *edit_fakeLightsMinIter;
+	GtkWidget *edit_fakeLightsMaxIter;
 
 	GtkWidget *label_animationFrame;
 	GtkWidget *label_animationSpeed;
@@ -581,6 +608,8 @@ struct sInterface
 	GtkWidget *label_OpenClMemorySize;
 	GtkWidget *label_OpenClWorkgroupSize;
 	GtkWidget *label_OpenClStatus;
+	GtkWidget *label_serverStatus;
+	GtkWidget *label_clientStatus;
 
 	GtkWidget *comboFractType;
 	GtkWidget *comboImageFormat;
@@ -601,6 +630,7 @@ struct sInterface
 	GtkWidget *checkLimits;
 	GtkWidget *checkSlowShading;
 	GtkWidget *checkBitmapBackground;
+	GtkWidget *checkBitmapBackgroundFulldome;
 	GtkWidget *checkAnimationSpeedDE;
 	GtkWidget *checkColoring;
 	GtkWidget *checkTgladMode;
@@ -644,6 +674,11 @@ struct sInterface
 	GtkWidget *checkPrimitiveWaterEnabled;
 	GtkWidget *checkOpenClEnable;
 	GtkWidget *checkIterFogEnable;
+	GtkWidget *checkNetRenderServerEnable;
+	GtkWidget *checkNetRenderServerScan;
+	GtkWidget *checkNetRenderClientEnable;
+	GtkWidget *checkPrimitiveOnlyPlane;
+	GtkWidget *checkFakeLightsEnabled;;
 
 	GtkWidget *colorSelectionGlow1;
 	GtkWidget *colorSelectionGlow2;
@@ -765,8 +800,11 @@ struct sNoGUIdata
 	bool playMode;
 	bool keyframeMode;
 	bool lowMemMode;
+	bool netrenderMode;
 	int startFrame;
 	int endFrame;
+	char netRenderIPString[20];
+	char netRenderPortString[6];
 	enumImageFormat imageFormat;
 	sParamRender fractparams;
 	std::vector<const char *> overrideStrings;
