@@ -100,7 +100,7 @@ void GetPaletteFromString(sRGB *palette, const char *paletteString)
 	for (int i = 0; i < 2000; i++)
 	{
 		int colour = 0;
-		sscanf(&paletteString[i], "%x", &colour);
+		sscanf(&paletteString[i], "%x", (unsigned int*)&colour);
 		sRGB rgbColour;
 		rgbColour.R = colour / 65536;
 		rgbColour.G = (colour / 256) % 256;
@@ -152,7 +152,7 @@ void SaveSettings(const char *filename, const sParamRender& params, bool compare
 	fprintfDot(fileSettings, "perspective", params.doubles.persp, 0.5, compare);
 	fprintfInt(fileSettings, "formula", params.fractal.formula, trig_optim, compare);
 	fprintfDot(fileSettings, "power", params.fractal.doubles.power, 9.0, compare);
-	fprintfInt(fileSettings, "N", params.fractal.N, 250, compare);
+	fprintfInt(fileSettings, "N", params.fractal.doubles.N, 250, compare);
 	fprintfInt(fileSettings, "minN", params.fractal.minN, 1, compare);
 	fprintfDot(fileSettings, "fractal_constant_factor", params.fractal.doubles.constantFactor, 1.0, compare);
 	fprintfDot(fileSettings, "quality", params.doubles.quality, 1.0, compare);
@@ -209,6 +209,7 @@ void SaveSettings(const char *filename, const sParamRender& params, bool compare
 	fprintfInt(fileSettings, "background_color_1_G", params.background_color1.G, 38306, compare);
 	fprintfInt(fileSettings, "background_color_1_B", params.background_color1.B, 65535, compare);
 	fprintfInt(fileSettings, "textured_background", params.textured_background, false, compare);
+	fprintfInt(fileSettings, "background_as_fuldome", params.background_as_fulldome, false, compare);
 	fprintfInt(fileSettings, "shadows_enabled", params.shadow, true, compare);
 	fprintfInt(fileSettings, "ambient_occlusion_enabled", params.global_ilumination, false, compare);
 	fprintfInt(fileSettings, "fast_ambient_occlusion_mode", params.fastGlobalIllumination, false, compare);
@@ -349,6 +350,7 @@ void SaveSettings(const char *filename, const sParamRender& params, bool compare
 	}
 
 	fprintfInt(fileSettings, "fish_eye", params.perspectiveType, false, compare);
+	fprintfInt(fileSettings, "fish_eye_180cut", params.fishEyeCut, false, compare);
 	fprintfDot(fileSettings, "stereo_eye_distance", params.doubles.stereoEyeDistance, 0.1, compare);
 	fprintfInt(fileSettings, "stereo_enabled", params.stereoEnabled, false, compare);
 
@@ -429,6 +431,8 @@ void SaveSettings(const char *filename, const sParamRender& params, bool compare
 	fprintfDot(fileSettings, "iteration_fog_opacity", params.doubles.iterFogOpacity, 1000.0, compare);
 	fprintfDot(fileSettings, "iteration_fog_opacity_trim", params.doubles.iterFogOpacityTrim, 4.0, compare);
 
+	fprintfInt(fileSettings, "primitive_only_plane", params.fractal.primitives.onlyPlane, false, compare);
+
 	if (!compare || params.fractal.primitives.planeEnable)
 	{
 		fprintfInt(fileSettings, "primitive_plane_enabled", params.fractal.primitives.planeEnable, false, compare);
@@ -508,6 +512,19 @@ void SaveSettings(const char *filename, const sParamRender& params, bool compare
 		fprintfInt(fileSettings, "primitive_water_colour_B", params.primitiveWaterColour.B, 10000, compare);
 	}
 
+	if (!compare || params.fakeLightsEnabled)
+	{
+		fprintfInt(fileSettings, "fake_lights_enabled", params.fakeLightsEnabled, false, compare);
+		fprintfDot(fileSettings, "fake_lights_intensity", params.doubles.fakeLightsIntensity, 1.0, compare);
+		fprintfDot(fileSettings, "fake_lights_visibility", params.doubles.fakeLightsVisibility, 1.0, compare);
+		fprintfDot(fileSettings, "fake_lights_visibility_size", params.doubles.fakeLightsVisibilitySize, 1.0, compare);
+		fprintfDot(fileSettings, "fake_lights_orbit_trap_X", params.fractal.doubles.fakeLightsOrbitTrap.x, 2.0, compare);
+		fprintfDot(fileSettings, "fake_lights_orbit_trap_Y", params.fractal.doubles.fakeLightsOrbitTrap.y, 0.0, compare);
+		fprintfDot(fileSettings, "fake_lights_orbit_trap_Z", params.fractal.doubles.fakeLightsOrbitTrap.z, 0.0, compare);
+		fprintfInt(fileSettings, "fake_lights_min_iter", params.fractal.fakeLightsMinIter, 1, compare);
+		fprintfInt(fileSettings, "fake_lights_max_iter", params.fractal.fakeLightsMaxIter, 2, compare);
+	}
+
 	if(strcmp(filename,"settings/.clipboard"))
 	{
 		if(!compare)
@@ -718,7 +735,7 @@ bool LoadOneSetting(const char* str1, const char *str2, sParamRender &params, bo
 	else if (!strcmp(str1, "perspective")) params.doubles.persp = atof2(str2);
 	else if (!strcmp(str1, "formula")) params.fractal.formula = (enumFractalFormula) atoi(str2);
 	else if (!strcmp(str1, "power")) params.fractal.doubles.power = atof2(str2);
-	else if (!strcmp(str1, "N")) params.fractal.N = atoi(str2);
+	else if (!strcmp(str1, "N")) params.fractal.doubles.N = atoi(str2);
 	else if (!strcmp(str1, "minN")) params.fractal.minN = atoi(str2);
 	else if (!strcmp(str1, "fractal_constant_factor")) params.fractal.doubles.constantFactor = atof2(str2);
 	else if (!strcmp(str1, "quality")) params.doubles.quality = atof2(str2);
@@ -773,6 +790,7 @@ bool LoadOneSetting(const char* str1, const char *str2, sParamRender &params, bo
 	else if (!strcmp(str1, "fog_colour_3_G")) params.fogColour3.G = atoi(str2);
 	else if (!strcmp(str1, "fog_colour_3_B")) params.fogColour3.B = atoi(str2);
 	else if (!strcmp(str1, "textured_background")) params.textured_background = atoi(str2);
+	else if (!strcmp(str1, "background_as_fuldome")) params.background_as_fulldome = atoi(str2);
 	else if (!strcmp(str1, "shadows_enabled")) params.shadow = atoi(str2);
 	else if (!strcmp(str1, "ambient_occlusion_enabled")) params.global_ilumination = atoi(str2);
 	else if (!strcmp(str1, "fast_ambient_occlusion_mode")) params.fastGlobalIllumination = atoi(str2);
@@ -864,6 +882,7 @@ bool LoadOneSetting(const char* str1, const char *str2, sParamRender &params, bo
   else if (!strcmp(str1, "hybrid_cyclic")) params.fractal.hybridCyclic = atoi(str2);
 
 	else if (!strcmp(str1, "fish_eye")) params.perspectiveType = (enumPerspectiveType)atoi(str2);
+	else if (!strcmp(str1, "fish_eye_180cut")) params.fishEyeCut = atoi(str2);
 
 	else if (!strcmp(str1, "stereo_enabled")) params.stereoEnabled = atoi(str2);
 	else if (!strcmp(str1, "stereo_eye_distance")) params.doubles.stereoEyeDistance = atof2(str2);
@@ -922,6 +941,8 @@ bool LoadOneSetting(const char* str1, const char *str2, sParamRender &params, bo
 	else if (!strcmp(str1, "iteration_fog_opacity")) params.doubles.iterFogOpacity = atof2(str2);
 	else if (!strcmp(str1, "iteration_fog_opacity_trim")) params.doubles.iterFogOpacityTrim = atof2(str2);
 
+	else if (!strcmp(str1, "primitive_only_plane")) params.fractal.primitives.onlyPlane = atoi(str2);
+
 	else if (!strcmp(str1, "primitive_plane_enabled")) params.fractal.primitives.planeEnable = atoi(str2);
 	else if (!strcmp(str1, "primitive_plane_centre_X")) params.fractal.doubles.primitives.planeCentre.x = atof2(str2);
 	else if (!strcmp(str1, "primitive_plane_centre_Y")) params.fractal.doubles.primitives.planeCentre.y = atof2(str2);
@@ -983,6 +1004,16 @@ bool LoadOneSetting(const char* str1, const char *str2, sParamRender &params, bo
 	else if (!strcmp(str1, "primitive_water_colour_G")) params.primitiveWaterColour.G = atoi(str2);
 	else if (!strcmp(str1, "primitive_water_colour_B")) params.primitiveWaterColour.B = atoi(str2);
 
+	else if (!strcmp(str1, "fake_lights_enabled")) params.fakeLightsEnabled = atoi(str2);
+	else if (!strcmp(str1, "fake_lights_min_iter")) params.fractal.fakeLightsMinIter = atoi(str2);
+	else if (!strcmp(str1, "fake_lights_max_iter")) params.fractal.fakeLightsMaxIter = atoi(str2);
+	else if (!strcmp(str1, "fake_lights_visibility")) params.doubles.fakeLightsVisibility = atof2(str2);
+	else if (!strcmp(str1, "fake_lights_visibility_size")) params.doubles.fakeLightsVisibilitySize = atof2(str2);
+	else if (!strcmp(str1, "fake_lights_intensity")) params.doubles.fakeLightsIntensity = atof2(str2);
+	else if (!strcmp(str1, "fake_lights_orbit_trap_X")) params.fractal.doubles.fakeLightsOrbitTrap.x = atof2(str2);
+	else if (!strcmp(str1, "fake_lights_orbit_trap_Y")) params.fractal.doubles.fakeLightsOrbitTrap.y = atof2(str2);
+	else if (!strcmp(str1, "fake_lights_orbit_trap_Z")) params.fractal.doubles.fakeLightsOrbitTrap.z = atof2(str2);
+
 	else if (!strcmp(str1, "file_destination")) strcpy(params.file_destination, str2);
 	else if (!strcmp(str1, "file_background")) strcpy(params.file_background, str2);
 	else if (!strcmp(str1, "file_envmap")) strcpy(params.file_envmap, str2);
