@@ -48,7 +48,7 @@ sShaderOutput MainShadow(sParamRender *param, sFractal *calcParam, CVector3 poin
 
 		if(param->imageSwitches.iterFogEnabled)
 		{
-			opacity = IterOpacity(dist * param->doubles.DE_factor,calcParam->itersOut, calcParam->N, param->doubles.iterFogOpacityTrim, param->doubles.iterFogOpacity);
+			opacity = IterOpacity(dist * param->doubles.DE_factor,calcParam->itersOut, calcParam->doubles.N, param->doubles.iterFogOpacityTrim, param->doubles.iterFogOpacity);
 		}
 		else
 		{
@@ -140,7 +140,7 @@ sShaderOutput AmbientOcclusion(sParamRender *param, sFractal *calcParam, CVector
 
 			if (param->imageSwitches.iterFogEnabled)
 			{
-				opacity = IterOpacity(dist * param->doubles.DE_factor, calcParam->itersOut, calcParam->N, param->doubles.iterFogOpacityTrim, param->doubles.iterFogOpacity);
+				opacity = IterOpacity(dist * param->doubles.DE_factor, calcParam->itersOut, calcParam->doubles.N, param->doubles.iterFogOpacityTrim, param->doubles.iterFogOpacity);
 			}
 			else
 			{
@@ -182,7 +182,7 @@ CVector3 CalculateNormals(sParamRender *param, sFractal *calcParam, CVector3 poi
 		if(calcParam->interiorMode) delta = dist_thresh * 0.2;
 
 		double s1, s2, s3, s4;
-		calcParam->N = param->fractal.N * 2;
+		calcParam->doubles.N = param->fractal.doubles.N * 2;
 		calcParam->minN = 0;
 
 		s1 = CalculateDistance(point, *calcParam);
@@ -199,7 +199,7 @@ CVector3 CalculateNormals(sParamRender *param, sFractal *calcParam, CVector3 poi
 		normal.x = s2 - s1;
 		normal.y = s3 - s1;
 		normal.z = s4 - s1;
-		calcParam->N = param->fractal.N;
+		calcParam->doubles.N = param->fractal.doubles.N;
 		calcParam->minN = param->fractal.minN;
 		//calcParam->DE_threshold = DEthreshold_temp;
 	}
@@ -306,9 +306,9 @@ sShaderOutput EnvMapping(CVector3 normal, CVector3 viewVector, cTexture *texture
 
 int SurfaceColour(sFractal *calcParam, CVector3 point)
 {
-	calcParam->N *= 10;
+	calcParam->doubles.N *= 10;
 	int nrKol = floor(Compute<colouring> (point, *calcParam));
-	calcParam->N /= 10;
+	calcParam->doubles.N /= 10;
 	nrKol = abs(nrKol) % (248*256);
 	return nrKol;
 }
@@ -318,14 +318,40 @@ sShaderOutput TexturedBackground(sParamRender *param, CVector3 viewVector)
 	sShaderOutput pixel2;
 	if (param->textured_background)
 	{
-		double alfaTexture = fmod(viewVector.GetAlfa() + 2.5 * M_PI, 2 * M_PI);
-		double betaTexture = viewVector.GetBeta();
-		if (betaTexture > 0.5 * M_PI) betaTexture = 0.5 * M_PI - betaTexture;
-		if (betaTexture < -0.5 * M_PI) betaTexture = -0.5 * M_PI + betaTexture;
-		double texX = alfaTexture / (2.0 * M_PI) * param->backgroundTexture->Width();
-		double texY = (betaTexture / (M_PI) + 0.5) * param->backgroundTexture->Height();
-		sRGB8 pixel = param->backgroundTexture->Pixel(texX, texY);
-		pixel2 = (sShaderOutput){ pixel.R/256.0, pixel.G/256.0, pixel.B/256.0};
+		if(param->background_as_fulldome)
+		{
+			double alfaTexture = viewVector.GetAlfa();
+			double betaTexture = viewVector.GetBeta();
+			int texWidth = param->backgroundTexture->Width()*0.5;
+			int texHeight = param->backgroundTexture->Height();
+			int offset = 0;
+
+			if(betaTexture < 0)
+			{
+				betaTexture = -betaTexture;
+				alfaTexture += M_PI;
+				offset = texWidth;
+			}
+			double texX = 0.5 * texWidth + cos(alfaTexture) * (1.0 - betaTexture / (0.5 * M_PI)) * texWidth * 0.5 + offset;
+			double texY = 0.5 * texHeight + sin(alfaTexture) * (1.0 - betaTexture / (0.5 * M_PI)) * texHeight * 0.5;
+			sRGB8 pixel = param->backgroundTexture->Pixel(texX, texY);
+			pixel2.R = pixel.R / 256.0;
+			pixel2.G = pixel.G / 256.0;
+			pixel2.B = pixel.B / 256.0;
+		}
+		else
+		{
+			double alfaTexture = fmod(viewVector.GetAlfa() + 2.5 * M_PI, 2 * M_PI);
+			double betaTexture = viewVector.GetBeta();
+			if (betaTexture > 0.5 * M_PI) betaTexture = 0.5 * M_PI - betaTexture;
+			if (betaTexture < -0.5 * M_PI) betaTexture = -0.5 * M_PI + betaTexture;
+			double texX = alfaTexture / (2.0 * M_PI) * param->backgroundTexture->Width();
+			double texY = (betaTexture / (M_PI) + 0.5) * param->backgroundTexture->Height();
+			sRGB8 pixel = param->backgroundTexture->Pixel(texX, texY);
+			pixel2.R = pixel.R/256.0;
+			pixel2.G = pixel.G/256.0;
+			pixel2.B = pixel.B/256.0;
+		}
 	}
 	else
 	{
@@ -349,7 +375,9 @@ sShaderOutput TexturedBackground(sParamRender *param, CVector3 viewVector)
 			pixel.B = (param->background_color2.B * Ngrad + param->background_color1.B * grad);
 		}
 
-		pixel2 = (sShaderOutput){ pixel.R/65536.0, pixel.G/65536.0, pixel.B/65536.0};
+		pixel2.R = pixel.R/65536.0;
+		pixel2.G = pixel.G/65536.0;
+		pixel2.B = pixel.B/65536.0;
 	}
 	return pixel2;
 }
@@ -461,7 +489,7 @@ double AuxShadow(sParamRender *fractParams, sFractal *calcParam, double wsp_pers
 
 		if(fractParams->imageSwitches.iterFogEnabled)
 		{
-			opacity = IterOpacity(dist * stepFactor * fractParams->doubles.DE_factor, calcParam->itersOut, calcParam->N, fractParams->doubles.iterFogOpacityTrim, fractParams->doubles.iterFogOpacity);
+			opacity = IterOpacity(dist * stepFactor * fractParams->doubles.DE_factor, calcParam->itersOut, calcParam->doubles.N, fractParams->doubles.iterFogOpacityTrim, fractParams->doubles.iterFogOpacity);
 		}
 		else
 		{
@@ -491,7 +519,8 @@ sShaderOutput VolumetricLight(sParamRender *param, sFractal *calcParam, CVector3
 {
 	//volumetric light
 	double tempDist = last_distance;
-	sShaderOutput volFog = (sShaderOutput){ 0.0,0.0,0.0};
+	sShaderOutput volFog;
+	volFog.R = volFog.G = volFog.B = 0.0;
 	enumPerspectiveType perspectiveType = param->perspectiveType;
 	double fov = param->doubles.persp;
 	double quality = param->doubles.quality;
@@ -543,7 +572,8 @@ sShaderOutput VolumetricLight(sParamRender *param, sFractal *calcParam, CVector3
 
 sShaderOutput VolumetricFog(sParamRender *param, int buffCount, double *distanceBuff, double *stepBuff, double *densityOut)
 {
-	sShaderOutput volFog = (sShaderOutput){ 0.0,0.0,0.0};
+	sShaderOutput volFog;
+	volFog.R = volFog.G = volFog.B = 0.0;
 	double density = 0;
 	double fogR = param->fogColour1.R;
 	double fogG = param->fogColour1.G;
@@ -552,7 +582,7 @@ sShaderOutput VolumetricFog(sParamRender *param, int buffCount, double *distance
 	double colourThresh2 = param->doubles.fogColour2Distance;
 	double fogReduce = param->doubles.fogDistanceFactor;
 	double fogIntensity = param->doubles.fogDensity;
-	for (int i = buffCount - 1; i >= 0; i--)
+	for (int i = buffCount - 2; i >= 0; i--)
 	{
 		double densityTemp = (stepBuff[i] * fogReduce) / (distanceBuff[i] * distanceBuff[i] + fogReduce * fogReduce);
 
@@ -579,9 +609,9 @@ sShaderOutput VolumetricFog(sParamRender *param, int buffCount, double *distance
 		density += densityTemp;
 	}
 	density *= fogIntensity;
-	volFog.R = fogR;
-	volFog.G = fogG;
-	volFog.B = fogB;
+	volFog.R = fogR/10.0;
+	volFog.G = fogG/10.0;
+	volFog.B = fogB/10.0;
 
 	*densityOut = density;
 	return volFog;
@@ -850,11 +880,11 @@ double IterOpacity(double step, double iters, double maxN, double trim, double o
 }
 
 sShaderOutput IterationFog(sParamRender *param, sFractal *calcParam, CVector3 point, double yStart, double min_y, double last_distance, double zoom,
-		CVector3 lightVector, bool found, double *densityOut, sVectorsAround *vectorsAround, int vectorsCount)
+		CVector3 lightVector, bool found, double *densityOut, sVectorsAround *vectorsAround, int vectorsCount, sRGB16 oldPixel)
 {
 	double density = 0.0;
 	double tempDist = last_distance;
-	sShaderOutput iterFog = { 0.0, 0.0, 0.0 };
+	sShaderOutput iterFog = {oldPixel.R/65536.0, oldPixel.G/65536.0, oldPixel.B/65536.0};
 	enumPerspectiveType perspectiveType = param->perspectiveType;
 	double fov = param->doubles.persp;
 	double quality = param->doubles.quality;
@@ -876,12 +906,30 @@ sShaderOutput IterationFog(sParamRender *param, sFractal *calcParam, CVector3 po
 		tempDist = tempDist * (1.0 - Random(1000) / 2000.0);
 
 		int L = calcParam->itersOut;
-		double opacity = IterOpacity(tempDist, L, param->fractal.N, param->doubles.iterFogOpacityTrim, param->doubles.iterFogOpacity);
+		double opacity = IterOpacity(tempDist, L, param->fractal.doubles.N, param->doubles.iterFogOpacityTrim, param->doubles.iterFogOpacity);
 
 		sShaderOutput newColour = { 0.0, 0.0, 0.0 };
 
 		if (opacity > 0)
 		{
+			//fog colour
+			double iterFactor = (double)2.0* (L - param->doubles.iterFogOpacityTrim)/(param->fractal.doubles.N - param->doubles.iterFogOpacityTrim);
+			double k = iterFactor;
+			if (k > 1) k = 1.0;
+			double kn = 1.0 - k;
+			double fogColR = (param->fogColour1.R * kn + param->fogColour2.R * k);
+			double fogColG = (param->fogColour1.G * kn + param->fogColour2.G * k);
+			double fogColB = (param->fogColour1.B * kn + param->fogColour2.B * k);
+
+			double k2 = iterFactor - 1.0;
+			if (k2 < 0.0) k2 = 0.0;
+			if (k2 > 1.0) k2 = 1.0;
+			kn = 1.0 - k2;
+			fogColR = (fogColR * kn + param->fogColour3.R * k2);
+			fogColG = (fogColG * kn + param->fogColour3.G * k2);
+			fogColB = (fogColB * kn + param->fogColour3.B * k2);
+			//----
+
 			for (int i = 0; i < 5; i++)
 			{
 				if (i == 0)
@@ -922,11 +970,17 @@ sShaderOutput IterationFog(sParamRender *param, sFractal *calcParam, CVector3 po
 				newColour.B += AO.B * param->doubles.imageAdjustments.globalIlum;
 			}
 
-			iterFog.R = iterFog.R * (1.0 - opacity) + newColour.R * opacity;
-			iterFog.G = iterFog.G * (1.0 - opacity) + newColour.G * opacity;
-			iterFog.B = iterFog.B * (1.0 - opacity) + newColour.B * opacity;
+			if(opacity > 1.0) opacity = 1.0;
+			{
+				iterFog.R = iterFog.R * (1.0 - opacity) + newColour.R * opacity * fogColR/65536.0;
+				iterFog.G = iterFog.G * (1.0 - opacity) + newColour.G * opacity * fogColG/65536.0;
+				iterFog.B = iterFog.B * (1.0 - opacity) + newColour.B * opacity * fogColB/65536.0;
+			}
 
 			density+=opacity;
+
+			//printf("iF.R=%g, dens=%g\n", iterFog.R, density);
+
 			if(density>1.0) density = 1.0;
 		}
 	}
@@ -945,6 +999,53 @@ sShaderOutput IterationFog(sParamRender *param, sFractal *calcParam, CVector3 po
 	return iterFog;
 }
 
+sShaderOutput FakeLights(sParamRender *param, sFractal *calcParam, CVector3 point, CVector3 normal, CVector3 viewVector, double dist_thresh, sShaderOutput *fakeSpec)
+{
+	sShaderOutput fakeLights;
+	double delta = dist_thresh * param->doubles.smoothness;
+	double r = Compute<orbitTrap>(point, *calcParam);
+	double fakeLight = param->doubles.fakeLightsIntensity/(r*r + 1e-100);
+
+	CVector3 deltax(delta, 0.0, 0.0);
+	double rx = Compute<orbitTrap>(point + deltax, *calcParam);
+	CVector3 deltay(0.0, delta, 0.0);
+	double ry = Compute<orbitTrap>(point + deltay, *calcParam);
+	CVector3 deltaz(0.0, 0.0, delta);
+	double rz = Compute<orbitTrap>(point + deltaz, *calcParam);
+
+	CVector3 fakeLightNormal;
+	fakeLightNormal.x = r - rx;
+	fakeLightNormal.y = r - ry;
+	fakeLightNormal.z = r - rz;
+
+	if(fakeLightNormal.x == 0 && fakeLightNormal.y == 0 && fakeLightNormal.z == 0)
+	{
+		fakeLightNormal.x = 0.0;
+	}
+	else
+	{
+		fakeLightNormal.Normalize();
+	}
+	double fakeLight2 = fakeLight * normal.Dot(fakeLightNormal);
+	if(fakeLight2 < 0) fakeLight2 = 0;
+
+	fakeLights.R = fakeLight2;
+	fakeLights.G = fakeLight2;
+	fakeLights.B = fakeLight2;
+
+	CVector3 half = fakeLightNormal - viewVector;
+	half.Normalize();
+	double fakeSpecular = normal.Dot(half);
+	if (fakeSpecular < 0.0) fakeSpecular = 0.0;
+	fakeSpecular = pow(fakeSpecular, 30.0) * fakeLight;
+	if (fakeSpecular > 15.0) fakeSpecular = 15.0;
+	fakeSpec->R = fakeSpecular;
+	fakeSpec->G = fakeSpecular;
+	fakeSpec->B = fakeSpecular;
+
+	return fakeLights;
+}
+
 /*
 void RenderBuddhabrot(sParamRender *fractParam)
 {
